@@ -10,7 +10,7 @@ use crate::{
 	api::convert_category_bitflags_to_url,
 	error::{Result, SponsorBlockError},
 	segment::{AcceptedCategories, ActionableSegmentKind, Segment},
-	util::{bool_from_integer_str, get_response_text, to_url_array},
+	util::{bool_from_integer_str, get_response_text, none_on_0_0_from_str, to_url_array},
 	Action,
 	AdditionalSegmentInfo,
 	Client,
@@ -46,52 +46,66 @@ struct RawSegment {
 	#[serde(deserialize_with = "bool_from_integer_str")]
 	locked: bool,
 	votes: i32,
-	#[serde(rename = "videoDuration")]
-	video_duration_upon_submission: f32,
+	#[serde(rename = "videoDuration", deserialize_with = "none_on_0_0_from_str")]
+	video_duration_upon_submission: Option<f32>,
 	#[serde(flatten)]
 	additional_info: AdditionalSegmentInfo,
 }
 
 impl RawSegment {
-	fn convert_to_segment(raw: Self, additional_info: bool) -> Result<Segment> {
-		let time_points = if let Some(points) = raw.time_points {
-			(points[0], points[1])
+	/// Converts a raw segment that more closely matches the structure returned
+	/// by the API to the proper rusty [`Segment`] type.
+	///
+	/// `additional_info` determines whether or not to include
+	/// `RawSegment.additional_info`, since it is always populated by Serde but
+	/// not with useful values under certain circumstances.
+	fn convert_to_segment(self, additional_info: bool) -> Result<Segment> {
+		let time_points = if let Some(points) = self.time_points {
+			points
 		} else {
-			(
-				raw.start_time
+			[
+				self.start_time
 					.expect("time_points was empty but so is start_time"),
-				raw.end_time
+				self.end_time
 					.expect("time_points was empty but so is end_time"),
-			)
+			]
 		};
-		if time_points.0 > time_points.1 {
+		if time_points[0] > time_points[1] {
 			return Err(SponsorBlockError::BadData(format!(
 				"segment start ({}) > end ({})",
-				time_points.0, time_points.1
+				time_points[0], time_points[1]
 			)));
 		}
-		if time_points.0 < 0.0 {
+		if time_points[0] < 0.0 {
 			return Err(SponsorBlockError::BadData(format!(
 				"segment start ({}) < 0",
-				time_points.0
+				time_points[0]
 			)));
 		}
-		if time_points.1 < 0.0 {
+		if time_points[1] < 0.0 {
 			return Err(SponsorBlockError::BadData(format!(
 				"segment end ({}) < 0",
-				time_points.1
+				time_points[1]
 			)));
 		}
+		if let Some(video_duration_upon_submission) = self.video_duration_upon_submission {
+			if video_duration_upon_submission < 0.0 {
+				return Err(SponsorBlockError::BadData(format!(
+					"video duration upon submission ({}) < 0",
+					video_duration_upon_submission
+				)));
+			}
+		}
 
 		Ok(Segment {
-			segment: raw.category.to_actionable_segment(time_points),
-			action_type: raw.action_type,
-			uuid: raw.uuid,
-			locked: raw.locked,
-			votes: raw.votes,
-			video_duration_on_submission: raw.video_duration_upon_submission,
+			segment: self.category.to_actionable_segment(time_points),
+			action_type: self.action_type,
+			uuid: self.uuid,
+			locked: self.locked,
+			votes: self.votes,
+			video_duration_on_submission: self.video_duration_upon_submission,
 			additional_info: if additional_info {
-				Some(raw.additional_info)
+				Some(self.additional_info)
 			} else {
 				None
 			},
@@ -207,7 +221,7 @@ impl Client {
 
 		video_segments
 			.drain(..)
-			.map(|s| RawSegment::convert_to_segment(s, false))
+			.map(|s| s.convert_to_segment(false))
 			.collect()
 	}
 
@@ -259,7 +273,7 @@ impl Client {
 		// Deserialize the response and parse it into the output
 		from_json_str::<Vec<RawSegment>>(response.as_str())?
 			.drain(..)
-			.map(|s| RawSegment::convert_to_segment(s, true))
+			.map(|s| s.convert_to_segment(true))
 			.collect()
 	}
 }
